@@ -56,7 +56,6 @@ import cx.ring.utils.Log;
 import cx.ring.utils.Ringer;
 import io.reactivex.Completable;
 
-
 public class HardwareServiceImpl extends HardwareService implements AudioManager.OnAudioFocusChangeListener, BluetoothWrapper.BluetoothChangeListener {
 
     private static final Point VIDEO_SIZE_LOW = new Point(320, 240);
@@ -171,36 +170,39 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
     synchronized public void updateAudioState(final SipCall.CallStatus state, final boolean incomingCall, final boolean isOngoingVideo) {
         Log.d(TAG, "updateAudioState: Call state updated to " + state + " Call is incoming: " + incomingCall + " Call is video: " + isOngoingVideo);
         boolean callEnded = state.equals(CallStatus.HUNGUP) || state.equals(CallStatus.FAILURE) || state.equals(CallStatus.OVER);
-        if (mBluetoothWrapper == null && !callEnded) {
-            mBluetoothWrapper = new BluetoothWrapper(mContext, this);
-        }
-        switch (state) {
-            case RINGING:
-                if (incomingCall)
-                    startRinging();
-                getFocus(RINGTONE_REQUEST);
-                if (incomingCall) {
-                    // ringtone for incoming calls
-                    mAudioManager.setMode(AudioManager.MODE_RINGTONE);
-                    setAudioRouting(true);
-                    mShouldSpeakerphone = isOngoingVideo;
-                }
-                else
+        try {
+            if (mBluetoothWrapper == null && !callEnded) {
+                mBluetoothWrapper = new BluetoothWrapper(mContext, this);
+            }
+            switch (state) {
+                case RINGING:
+                    if (incomingCall)
+                        startRinging();
+                    getFocus(RINGTONE_REQUEST);
+                    if (incomingCall) {
+                        // ringtone for incoming calls
+                        mAudioManager.setMode(AudioManager.MODE_RINGTONE);
+                        setAudioRouting(true);
+                        mShouldSpeakerphone = isOngoingVideo;
+                    } else
+                        setAudioRouting(isOngoingVideo);
+                    break;
+                case CURRENT:
+                    stopRinging();
+                    getFocus(CALL_REQUEST);
+                    mAudioManager.setMode(AudioManager.MODE_IN_COMMUNICATION);
                     setAudioRouting(isOngoingVideo);
-                break;
-            case CURRENT:
-                stopRinging();
-                getFocus(CALL_REQUEST);
-                mAudioManager.setMode(AudioManager.MODE_IN_COMMUNICATION);
-                setAudioRouting(isOngoingVideo);
-                break;
-            case HOLD:
-            case UNHOLD:
-            case INACTIVE:
-                break;
-            default:
-                closeAudioState();
-                break;
+                    break;
+                case HOLD:
+                case UNHOLD:
+                case INACTIVE:
+                    break;
+                default:
+                    closeAudioState();
+                    break;
+            }
+        } catch (Exception e) {
+            Log.e(TAG, "Error updating audio state", e);
         }
     }
 
@@ -408,7 +410,7 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
     @Override
     public void getCameraInfo(String camId, IntVect formats, UintVect sizes, UintVect rates) {
         // Use a larger resolution for Android 6.0+, 64 bits devices
-        final boolean useLargerSize = Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && Build.SUPPORTED_64_BIT_ABIS.length > 0;
+        final boolean useLargerSize = Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && (Build.SUPPORTED_64_BIT_ABIS.length > 0 || mPreferenceService.isHardwareAccelerationEnabled());
         //int MIN_WIDTH = useLargerSize ? (useHD ? VIDEO_WIDTH_HD : VIDEO_WIDTH) : VIDEO_WIDTH_MIN;
         Point minVideoSize;
         if (useLargerSize)
@@ -441,12 +443,12 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
         cameraService.setParameters(camId, format, width, height, rate, windowManager.getDefaultDisplay().getRotation());
     }
 
-    public void startScreenShare(Object projection) {
+    public boolean startScreenShare(Object projection) {
         MediaProjection mediaProjection = (MediaProjection) projection;
         if (mIsCapturing) {
             endCapture();
         }
-        if (!mIsScreenSharing) {
+        if (!mIsScreenSharing && mediaProjection != null) {
             mIsScreenSharing = true;
             mediaProjection.registerCallback(new MediaProjection.Callback(){
                 @Override
@@ -454,7 +456,14 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
                     stopScreenShare();
                 }
             }, cameraService.getVideoHandler());
-            cameraService.startScreenSharing(mediaProjection, mContext.getResources().getDisplayMetrics());
+            if (!cameraService.startScreenSharing(mediaProjection, mContext.getResources().getDisplayMetrics())) {
+                mIsScreenSharing = false;
+                mediaProjection.stop();
+                return false;
+            }
+            return true;
+        } else {
+            return false;
         }
     }
 
@@ -469,6 +478,10 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
 
     @Override
     public void startCapture(@Nullable String camId) {
+        if (mIsScreenSharing) {
+            cameraService.stopScreenSharing();
+            mIsScreenSharing = false;
+        }
         mShouldCapture = true;
         if (mIsCapturing && mCapturingId != null && mCapturingId.equals(camId)) {
             return;
@@ -535,6 +548,10 @@ public class HardwareServiceImpl extends HardwareService implements AudioManager
         Log.d(TAG, "stopCapture: " + cameraService.isOpen());
         mShouldCapture = false;
         endCapture();
+        if (mIsScreenSharing) {
+            cameraService.stopScreenSharing();
+            mIsScreenSharing = false;
+        }
     }
 
     public void requestKeyFrame() {
