@@ -63,23 +63,41 @@ public class AndroidFileUtils {
     private final static int ORIENTATION_RIGHT = 90;
     private final static int MAX_IMAGE_DIMENSION = 1024;
 
-    public static boolean copyAssetFolder(AssetManager assetManager, String fromAssetPath, File toPath) {
+    /**
+     * Copy assets from a folder recursively ( files and subfolder)
+     * @param assetManager Asset Manager ( you can get it from Context.getAssets() )
+     * @param fromAssetPath path to the assets folder we want to copy
+     * @param toPath a directory in internal storage
+     * @return true if success
+     */
+    public static boolean copyAssetFolder( @NonNull AssetManager assetManager, String fromAssetPath, File toPath) {
         try {
-            String[] files = assetManager.list(fromAssetPath);
-            if (!toPath.exists()) {
-                toPath.mkdirs();
-            }
             boolean res = true;
-            for (String file : files) {
-                File newFile = new File(toPath, file);
-                if (file.contains("")) {
-                    Log.d(TAG, "Copying file :" + fromAssetPath + File.separator + file + " to " + newFile);
-                    res &= copyAsset(assetManager, fromAssetPath + File.separator + file, newFile);
-                } else {
-                    Log.d(TAG, "Copying folder :" + fromAssetPath + File.separator + file + " to " + newFile);
-                    res &= copyAssetFolder(assetManager, fromAssetPath + File.separator + file, newFile);
+
+            // mkdirs checks if the folder exists and if not creates it
+            toPath.mkdirs();
+
+            // List the files of this asset directory
+            String[] files = assetManager.list(fromAssetPath);
+
+            if(files != null){
+                for (String file : files) {
+
+                    String subAsset = fromAssetPath + File.separator + file;
+
+                    if(isAssetDirectory(assetManager, subAsset)){
+                        String destination = toPath.getAbsolutePath() + File.separator + file;
+                        File newDir = new File(destination);
+                        copyAssetFolder(assetManager, subAsset, newDir);
+                        Log.d(TAG, "Copied folder: " + subAsset + " to " + newDir);
+                    } else {
+                        File newFile = new File(toPath, file);
+                        res &= copyAsset(assetManager, fromAssetPath + File.separator + file, newFile);
+                        Log.d(TAG, "Copied file: " + subAsset + " to " + newFile);
+                    }
                 }
             }
+
             return res;
         } catch (IOException e) {
             Log.e(TAG, "Error while copying asset folder", e);
@@ -87,6 +105,59 @@ public class AndroidFileUtils {
         }
     }
 
+    /**
+     * Checks whether an asset is a file or a directory
+     * @param assetManager Asset Manager ( you can get it from Context.getAssets() )
+     * @param fromAssetPath  asset path, if just a file in  assets root folder then it should be
+     *                       the file name, otherwise, folder/filename
+     * @return boolean directory or not
+     */
+    public static boolean isAssetDirectory( @NonNull AssetManager assetManager, String fromAssetPath) {
+        boolean res = false;
+        try{
+            String[] files = assetManager.list(fromAssetPath);
+            if(files !=null) {
+                if(files.length > 0) {
+                    res = true;
+                }
+            }
+        } catch (IOException e) {
+            Log.e(TAG, "Error while reading an asset ", e);
+        }
+        return res;
+    }
+
+    /**
+     * Prints assets tree
+     * @param assetManager Asset Manager ( you can get it from Context.getAssets() )
+     * @param rootPath default empty, sub folder otherwise
+     * @param fileName the name of the file or folder
+     * @param level default 0, should be 0
+     */
+    public static void assetTree( @NonNull AssetManager assetManager, String rootPath, String fileName,  int level) {
+        try {
+            String fromAssetPath;
+            if(TextUtils.isEmpty(rootPath)) {
+                fromAssetPath = fileName;
+            } else {
+                fromAssetPath = rootPath + File.separator+ fileName;
+            }
+
+            String repeated = new String(new char[level]).replace("\0", "\t|");
+
+            String[] files = assetManager.list(fromAssetPath);
+
+            if (files != null) {
+                Log.d(TAG, "|"+ repeated + "-- " + fileName);
+                for(String file : files) {
+                    assetTree(assetManager,fromAssetPath,file,level+1);
+                }
+            }
+        } catch (IOException e) {
+            Log.e(TAG, "Error while reading asset ", e);
+        }
+    }
+
     private static boolean copyAsset(AssetManager assetManager, String fromAssetPath, File toPath) {
         InputStream in;
         OutputStream out;
@@ -201,13 +272,19 @@ public class AndroidFileUtils {
         return "application/octet-stream";
     }
 
+    public static File getTempShareDir(@NonNull Context context) {
+        File tmp = new File(context.getCacheDir(), "tmp");
+        tmp.mkdir();
+        return tmp;
+    }
+
     public static File createImageFile(@NonNull Context context) throws IOException {
         // Create an image file name
         String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss", Locale.US).format(new Date());
         String imageFileName = "img_" + timeStamp + "_";
 
         // Save a file: path for use with ACTION_VIEW intents
-        return File.createTempFile(imageFileName, ".jpg", context.getExternalCacheDir());
+        return File.createTempFile(imageFileName, ".jpg", getTempShareDir(context));
     }
 
     public static File createAudioFile(@NonNull Context context) throws IOException {
@@ -216,7 +293,7 @@ public class AndroidFileUtils {
         String imageFileName = "audio_" + timeStamp + "_";
 
         // Save a file: path for use with ACTION_VIEW intents
-        return File.createTempFile(imageFileName, ".mp3", context.getExternalCacheDir());
+        return File.createTempFile(imageFileName, ".mp3", getTempShareDir(context));
     }
     public static File createVideoFile(@NonNull Context context) throws IOException {
         // Create an image file name
@@ -224,9 +301,15 @@ public class AndroidFileUtils {
         String imageFileName = "video_" + timeStamp + "_";
 
         // Save a file: path for use with ACTION_VIEW intents
-        return File.createTempFile(imageFileName, ".webm", context.getExternalCacheDir());
+        return File.createTempFile(imageFileName, ".webm", getTempShareDir(context));
     }
 
+    /**
+     * Copies a file from a uri whether locally on a remote location to the local cache
+     * @param context Context to get access to cache directory
+     * @param uri uri of the
+     * @return Single<File> which points to the newly created copy in the cache
+     */
     public static @NonNull Single<File> getCacheFile(@NonNull Context context, @NonNull Uri uri) {
         ContentResolver contentResolver = context.getContentResolver();
         File cacheDir = context.getCacheDir();
