@@ -89,28 +89,33 @@ Call::Call(Account& account, const std::string& id, Call::CallType type,
                             Call::ConnectionState cnx_state,
                             UNUSED int code) {
 
-        if (cnx_state == ConnectionState::PROGRESSING) {
+        if (cnx_state == ConnectionState::PROGRESSING
+            && startFallback_.exchange(false)
+            && not isSubcall()) {
             // If the other peer lose the connectivity during the progressing
             // this means that the other peer had a connectivity change we didn't
             // detect for now.
             // In this case, we let two secs before sending a request via the DHT
             // just to bypass the CONNECTING status
 
-
-           std::weak_ptr<Call> callWkPtr = shared_from_this();
-           Manager::instance().scheduler().scheduleIn([callWkPtr]{
-               if (auto callShPtr = callWkPtr.lock()) {
+            std::weak_ptr<Call> callWkPtr = shared_from_this();
+            Manager::instance().scheduler().scheduleIn([callWkPtr]{
+                if (auto callShPtr = callWkPtr.lock()) {
                     if (callShPtr->getConnectionState() == Call::ConnectionState::PROGRESSING) {
-                         JAMI_WARN("Call %s is still connecting after timeout, sending fallback request",
-                             callShPtr->getCallId().c_str());
-                         if (callShPtr->onNeedFallback_) callShPtr->onNeedFallback_();
+                        JAMI_WARN("Call %s is still connecting after timeout, sending fallback request",
+                            callShPtr->getCallId().c_str());
+                        if (callShPtr->onNeedFallback_) callShPtr->onNeedFallback_();
                     }
                 }
-           }, std::chrono::seconds(2));
+            }, std::chrono::seconds(2));
         }
 
         checkPendingIM();
-        checkAudio();
+        std::weak_ptr<Call> callWkPtr = shared_from_this();
+        runOnMainThread([callWkPtr]{
+            if (auto call = callWkPtr.lock())
+                call->checkAudio();
+        });
 
         // if call just started ringing, schedule call timeout
         if (type_ == CallType::INCOMING and cnx_state == ConnectionState::RINGING) {
@@ -389,6 +394,12 @@ Call::getNullDetails()
 void
 Call::onTextMessage(std::map<std::string, std::string>&& messages)
 {
+    auto it = messages.find("application/confInfo+json");
+    if (it != messages.end()) {
+        setConferenceInfo(it->second);
+        return;
+    }
+
     {
         std::lock_guard<std::recursive_mutex> lk {callMutex_};
         if (parent_) {
@@ -435,10 +446,14 @@ Call::addSubCall(Call& subcall)
         subcall.sendTextMessage(msg.first, msg.second);
 
     subcall.addStateListener(
-        [&subcall](Call::CallState new_state, Call::ConnectionState new_cstate, UNUSED int code) {
-            auto parent = subcall.parent_;
-            assert(parent != nullptr); // subcall cannot be "un-parented"
-            parent->subcallStateChanged(subcall, new_state, new_cstate);
+        [sub = subcall.weak(), parent = weak()](Call::CallState new_state, Call::ConnectionState new_cstate, int code) {
+            runOnMainThread([sub, parent, new_state, new_cstate, code]() {
+                if (auto p = parent.lock()) {
+                    if (auto s = sub.lock()) {
+                      p->subcallStateChanged(*s, new_state, new_cstate);
+                    }
+                }
+            });
         });
 }
 
@@ -597,4 +612,32 @@ Call::safePopSubcalls()
     return old_value;
 }
 
+void
+Call::setConferenceInfo(const std::string& msg)
+{
+    ConfInfo newInfo;
+    Json::Value json;
+    std::string err;
+    Json::CharReaderBuilder rbuilder;
+    auto reader = std::unique_ptr<Json::CharReader>(rbuilder.newCharReader());
+    if (reader->parse(msg.data(), msg.data() + msg.size(), &json, &err)) {
+        for (const auto& participantInfo: json) {
+            ParticipantInfo pInfo;
+            if (!participantInfo.isMember("uri")) continue;
+            pInfo.fromJson(participantInfo);
+            newInfo.emplace_back(pInfo);
+        }
+    }
+
+    std::vector<std::map<std::string, std::string>> toSend;
+    {
+        std::lock_guard<std::mutex> lk(confInfoMutex_);
+        confInfo_ = std::move(newInfo);
+        toSend = confInfo_.toVectorMapStringString();
+    }
+
+    // Inform client that layout has changed
+    jami::emitSignal<DRing::CallSignal::OnConferenceInfosUpdated>(id_, std::move(toSend));
+}
+
 } // namespace jami
