@@ -16,7 +16,9 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 #include "namedirectory.h"
 
 #include "logger.h"
@@ -48,6 +50,7 @@ namespace jami {
 constexpr const char* const QUERY_NAME {"/name/"};
 constexpr const char* const QUERY_ADDR {"/addr/"};
 constexpr const char* const CACHE_DIRECTORY {"namecache"};
+constexpr const char DEFAULT_SERVER_HOST[] = "https://ns.jami.net";
 
 const std::string  HEX_PREFIX = "0x";
 constexpr std::chrono::seconds SAVE_INTERVAL {5};
@@ -66,6 +69,9 @@ toLower(std::string& string)
     std::transform(string.begin(), string.end(), string.begin(), ::tolower);
 }
 
+NameDirectory&
+NameDirectory::instance() { return instance(DEFAULT_SERVER_HOST); }
+
 void
 NameDirectory::lookupUri(const std::string& uri, const std::string& default_server,
                          LookupCallback cb)
@@ -90,6 +96,8 @@ NameDirectory::NameDirectory(const std::string& serverUrl, std::shared_ptr<dht::
    : serverUrl_(serverUrl), logger_(std::move(l))
    , httpContext_(Manager::instance().ioContext())
 {
+    if (!serverUrl_.empty() && serverUrl_.back() == '/')
+        serverUrl_.pop_back();
     resolver_ = std::make_shared<dht::http::Resolver>(*httpContext_, serverUrl, logger_);
     cachePath_ = fileutils::get_cache_dir() + DIR_SEPARATOR_STR + CACHE_DIRECTORY +
                                               DIR_SEPARATOR_STR + resolver_->get_url().host;
@@ -135,16 +143,12 @@ NameDirectory::lookupAddress(const std::string& addr, LookupCallback cb)
         cb(cacheResult, Response::found);
         return;
     }
-    auto request = std::make_shared<Request>(*httpContext_, resolver_, QUERY_ADDR + addr);
+    auto request = std::make_shared<Request>(*httpContext_, resolver_, serverUrl_ + QUERY_ADDR + addr);
     auto reqid = request->id();
     try {
-        request->set_connection_type(restinio::http_connection_header_t::keep_alive);
         request->set_method(restinio::http_method_get());
         setHeaderFields(*request);
-        request->add_on_state_change_callback([this, cb=std::move(cb), reqid, addr]
-                                              (Request::State state, const dht::http::Response& response){
-            if (state != Request::State::DONE)
-                return;
+        request->add_on_done_callback([this, cb=std::move(cb), reqid, addr] (const dht::http::Response& response){
             if (response.status_code >= 400 && response.status_code < 500){
                 cb("", Response::notFound);
             }
@@ -213,15 +217,13 @@ NameDirectory::lookupName(const std::string& n, LookupCallback cb)
         cb(cacheResult, Response::found);
         return;
     }
-    auto request = std::make_shared<Request>(*httpContext_, resolver_, QUERY_NAME + name);
+    auto request = std::make_shared<Request>(*httpContext_, resolver_, serverUrl_ + QUERY_NAME + name);
     auto reqid = request->id();
     try {
         request->set_method(restinio::http_method_get());
         setHeaderFields(*request);
-        request->add_on_state_change_callback([this, reqid, name, cb=std::move(cb)]
-                                              (Request::State state, const dht::http::Response& response){
-            if (state != Request::State::DONE)
-                return;
+        request->add_on_done_callback([this, reqid, name, cb=std::move(cb)]
+                                              (const dht::http::Response& response){
             if (response.status_code >= 400 && response.status_code < 500)
                 cb("", Response::notFound);
             else if (response.status_code < 200 || response.status_code > 299)
@@ -321,7 +323,7 @@ void NameDirectory::registerName(const std::string& addr, const std::string& n,
                     jami::Blob(publickey.begin(), publickey.end()))  << "\"}";
         body = ss.str();
     }
-    auto request = std::make_shared<Request>(*httpContext_, resolver_, QUERY_NAME + name);
+    auto request = std::make_shared<Request>(*httpContext_, resolver_, serverUrl_ + QUERY_NAME + name);
     auto reqid = request->id();
     try {
         request->set_method(restinio::http_method_post());
@@ -330,10 +332,7 @@ void NameDirectory::registerName(const std::string& addr, const std::string& n,
 
         JAMI_WARN("RegisterName: sending request %s %s", addr.c_str(), name.c_str());
 
-        request->add_on_state_change_callback([this, reqid, name, addr, cb=std::move(cb)]
-                                              (Request::State state, const dht::http::Response& response){
-            if (state != Request::State::DONE)
-                return;
+        request->add_on_done_callback([this, reqid, name, addr, cb=std::move(cb)](const dht::http::Response& response){
             if (response.status_code == 400){
                 cb(RegistrationResponse::incompleteRequest);
                 JAMI_ERR("RegistrationResponse::incompleteRequest");
@@ -387,7 +386,7 @@ void NameDirectory::registerName(const std::string& addr, const std::string& n,
 void
 NameDirectory::scheduleCacheSave()
 {
-    JAMI_DBG("Scheduling cache save to %s", cachePath_.c_str());
+    // JAMI_DBG("Scheduling cache save to %s", cachePath_.c_str());
     std::weak_ptr<Task> task = Manager::instance().scheduler().scheduleIn([this]{
         dht::ThreadPool::io().run([this] {
             saveCache();
