@@ -11,6 +11,7 @@
  *  Author: Guillaume Roguez <guillaume.roguez@savoirfairelinux.com>
  *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *  Author: Philippe Gorley <philippe.gorley@savoirfairelinux.com>
+ *  Author: Aline Gondim Santos <aline.gondimsantos@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,6 +42,8 @@
 #include "account.h"
 #include "string_utils.h"
 #include "jamidht/jamiaccount.h"
+#include "sip/sipvoiplink.h"
+#include "account.h"
 #include <opendht/rng.h>
 using random_device = dht::crypto::random_device;
 
@@ -63,6 +66,10 @@ using random_device = dht::crypto::random_device;
 #include "audio/sound/dtmf.h"
 #include "audio/ringbufferpool.h"
 
+#ifdef ENABLE_PLUGIN
+#include "plugin/jamipluginmanager.h"
+#endif
+
 #ifdef ENABLE_VIDEO
 #include "client/videomanager.h"
 #include "video/video_scaler.h"
@@ -77,6 +84,7 @@ using random_device = dht::crypto::random_device;
 
 #include "libav_utils.h"
 #include "video/sinkclient.h"
+#include "media/video/video_mixer.h"
 #include "audio/tonecontrol.h"
 
 #include "data_transfer.h"
@@ -287,7 +295,7 @@ struct Manager::ManagerPimpl
      * Play one tone
      * @return false if the driver is uninitialize
      */
-    void playATone(Tone::TONEID toneId);
+    void playATone(Tone::ToneId toneId);
 
     int getCurrentDeviceIndex(DeviceType type);
 
@@ -412,6 +420,12 @@ struct Manager::ManagerPimpl
 #ifdef ENABLE_VIDEO
     std::unique_ptr<VideoManager> videoManager_;
 #endif
+
+    std::unique_ptr<SIPVoIPLink> sipLink_;
+#ifdef ENABLE_PLUGIN
+    /* Jami Plugin Manager */
+    JamiPluginManager jami_plugin_manager;
+#endif
 };
 
 Manager::ManagerPimpl::ManagerPimpl(Manager& base)
@@ -464,7 +478,7 @@ Manager::ManagerPimpl::parseConfiguration()
  * Multi Thread
  */
 void
-Manager::ManagerPimpl::playATone(Tone::TONEID toneId)
+Manager::ManagerPimpl::playATone(Tone::ToneId toneId)
 {
     if (not base_.voipPreferences.getPlayTones())
         return;
@@ -522,11 +536,17 @@ Manager::ManagerPimpl::processRemainingParticipants(Conference& conf)
         auto p = participants.begin();
         if (auto call = base_.getCallFromCallID(*p)) {
             call->setConfId("");
-            // if we are not listening to this conference
-            if (current_call_id != conf.getConfID())
-                base_.onHoldCall(call->getCallId());
-            else
-                switchCall(call);
+            // if we are not listening to this conference and not a rendez-vous
+            auto isRdv = false;
+            if (auto acc = std::dynamic_pointer_cast<JamiAccount>(base_.getAccount(call->getAccountId())))
+                isRdv = acc->getAccountDetails()[Conf::CONFIG_ACCOUNT_ISRENDEZVOUS] == TRUE_STR;
+
+            if (!isRdv) {
+                if (current_call_id != conf.getConfID())
+                    base_.onHoldCall(call->getCallId());
+                else
+                    switchCall(call);
+            }
         }
 
         JAMI_DBG("No remaining participants, remove conference");
@@ -686,6 +706,9 @@ Manager::Manager()
     , shortcutPreferences()
 #ifdef ENABLE_VIDEO
     , videoPreferences()
+#endif
+#ifdef ENABLE_PLUGIN
+    , pluginPreferences()
 #endif
     , callFactory()
     , accountFactory()
@@ -741,6 +764,10 @@ Manager::init(const std::string &config_file)
 
     setDhtLogLevel();
 
+    // Manager can restart without being recreated (Unit tests)
+    // So only create the SipLink once
+    pimpl_->sipLink_ = std::make_unique<SIPVoIPLink>();
+
     check_rename(fileutils::get_cache_dir(PACKAGE_OLD), fileutils::get_cache_dir());
     check_rename(fileutils::get_data_dir(PACKAGE_OLD), fileutils::get_data_dir());
     check_rename(fileutils::get_config_dir(PACKAGE_OLD), fileutils::get_config_dir());
@@ -752,7 +779,7 @@ Manager::init(const std::string &config_file)
 
     bool no_errors = true;
 
-    // manager can restart without being recreated (android)
+    // manager can restart without being recreated (Unit tests)
     pimpl_->finished_ = false;
 
     try {
@@ -761,7 +788,6 @@ Manager::init(const std::string &config_file)
         JAMI_ERR("%s", e.what());
         no_errors = false;
     }
-
     // Some VoIP services support SIP/TLS and SRTP, but do not set the
     // correct schema in the INVITE request. For more details, see:
     // https://trac.pjsip.org/repos/ticket/1735
@@ -776,9 +802,6 @@ Manager::init(const std::string &config_file)
         // restore previous configuration
         JAMI_WARN("Restoring last working configuration");
 
-        // keep a reference to sipvoiplink while destroying the accounts
-        const auto sipvoiplink = getSIPVoIPLink();
-
         try {
             // remove accounts from broken configuration
             removeAccounts();
@@ -849,6 +872,12 @@ Manager::finish() noexcept
         // Also, it must be called before pj_shutdown to avoid any problem
         pimpl_->ice_tf_.reset();
 
+        // NOTE: sipLink_->shutdown() is needed because this will perform
+        // sipTransportBroker->shutdown(); which will call Manager::instance().sipVoIPLink()
+        // so the pointer MUST NOT be resetted at this point
+        pimpl_->sipLink_->shutdown();
+        pimpl_->sipLink_.reset();
+
         pj_shutdown();
 
         if (!pimpl_->ioContext_->stopped()){
@@ -871,7 +900,11 @@ Manager::isCurrentCall(const Call& call) const
 bool
 Manager::hasCurrentCall() const
 {
-    return not pimpl_->currentCall_.empty();
+    for (const auto& call: callFactory.getAllCalls()) {
+        if (!call->isSubcall() && call->getStateStr() == DRing::Call::StateEvent::CURRENT)
+            return true;
+    }
+    return false;
 }
 
 std::shared_ptr<Call>
@@ -949,6 +982,11 @@ Manager::answerCall(const std::string& call_id)
         return false;
     }
 
+    if (call->getConnectionState() != Call::ConnectionState::RINGING) {
+        // The call is already answered
+        return true;
+    }
+
     // If ringing
     stopTone();
 
@@ -1028,7 +1066,6 @@ Manager::hangupCall(const std::string& callId)
 
     try {
         call->hangup(0);
-        checkAudio();
     } catch (const VoipLinkException &e) {
         JAMI_ERR("%s", e.what());
         return false;
@@ -1064,29 +1101,29 @@ Manager::onHoldCall(const std::string& callId)
 
     if (auto call = getCallFromCallID(callId)) {
         try {
-            result = call->onhold();
-            if (result)
+            result = call->onhold([=](bool ok) {
+                if (!ok) {
+                    JAMI_ERR("hold failed for call %s", callId.c_str());
+                    return;
+                }
                 removeAudio(*call); // Unbind calls in main buffer
+                // Remove call from the queue if it was still there
+                pimpl_->removeWaitingCall(callId);
+
+                // keeps current call id if the action is not holding this call
+                // or a new outgoing call. This could happen in case of a conference
+                if (current_call_id == callId)
+                    pimpl_->unsetCurrentCall();
+            });
         } catch (const VoipLinkException &e) {
             JAMI_ERR("%s", e.what());
             result = false;
         }
-
     } else {
         JAMI_DBG("CallID %s doesn't exist in call onHold", callId.c_str());
         return false;
     }
 
-    if (result) {
-        // Remove call from the queue if it was still there
-        pimpl_->removeWaitingCall(callId);
-
-        // keeps current call id if the action is not holding this call
-        // or a new outgoing call. This could happen in case of a conference
-        if (current_call_id == callId)
-            pimpl_->unsetCurrentCall();
-    }
-
     return result;
 }
 
@@ -1103,21 +1140,24 @@ Manager::offHoldCall(const std::string& callId)
         return false;
 
     try {
-        result = call->offhold();
+        result = call->offhold([=](bool ok) {
+            if (!ok) {
+                JAMI_ERR("off hold failed for call %s", callId.c_str());
+                return;
+            }
+
+            if (isConferenceParticipant(callId))
+                pimpl_->switchCall(call->getConfId());
+            else
+                pimpl_->switchCall(call);
+
+            addAudio(*call);
+        });
     } catch (const VoipLinkException &e) {
         JAMI_ERR("%s", e.what());
         return false;
     }
 
-    if (result) {
-        if (isConferenceParticipant(callId))
-            pimpl_->switchCall(call->getConfId());
-        else
-            pimpl_->switchCall(call);
-
-        addAudio(*call);
-    }
-
     return result;
 }
 
@@ -1187,8 +1227,6 @@ Manager::refuseCall(const std::string& id)
 
     call->refuse();
 
-    checkAudio();
-
     pimpl_->removeWaitingCall(id);
 
     // Disconnect streams
@@ -1243,10 +1281,7 @@ bool
 Manager::holdConference(const std::string& id)
 {
     if (auto conf = getConferenceFromID(id)) {
-        for (const auto &item : conf->getParticipantList())
-            onHoldCall(item);
-
-        conf->setState(Conference::State::HOLD);
+        conf->detach();
         emitSignal<DRing::CallSignal::ConferenceChanged>(conf->getConfID(), conf->getStateStr());
         return true;
     }
@@ -1267,6 +1302,8 @@ Manager::unHoldConference(const std::string& id)
             conf->setState(Conference::State::ACTIVE_ATTACHED);
             emitSignal<DRing::CallSignal::ConferenceChanged>(conf->getConfID(), conf->getStateStr());
             return true;
+        } else if (conf->getState() == Conference::State::ACTIVE_DETACHED) {
+            pimpl_->addMainParticipant(*conf);
         }
     }
     return false;
@@ -1314,6 +1351,10 @@ Manager::addParticipant(const std::string& callId,
 
     pimpl_->bindCallToConference(*call, *conf);
 
+    // Don't attach current user yet
+    if (conf->getState() == Conference::State::ACTIVE_DETACHED)
+        return true;
+
     // TODO: remove this ugly hack => There should be different calls when double clicking
     // a conference to add main participant to it, or (in this case) adding a participant
     // toconference
@@ -1331,19 +1372,8 @@ Manager::ManagerPimpl::addMainParticipant(Conference& conf)
 {
     {
         std::lock_guard<std::mutex> lock(audioLayerMutex_);
-        for (const auto &item_p : conf.getParticipantList()) {
-            ringbufferpool_->bindCallID(item_p, RingBufferPool::DEFAULT_ID);
-            // Reset ringbuffer's readpointers
-            ringbufferpool_->flush(item_p);
-        }
-        ringbufferpool_->flush(RingBufferPool::DEFAULT_ID);
+        conf.attach();
     }
-
-    if (conf.getState() == Conference::State::ACTIVE_DETACHED)
-        conf.setState(Conference::State::ACTIVE_ATTACHED);
-    else
-        JAMI_WARN("Invalid conference state %d while adding main participant", (int)conf.getState());
-
     emitSignal<DRing::CallSignal::ConferenceChanged>(conf.getConfID(), conf.getStateStr());
     switchCall(conf.getConfID());
 }
@@ -1365,7 +1395,7 @@ Manager::getCallFromCallID(const std::string& callID) const
 }
 
 bool
-Manager::joinParticipant(const std::string& callId1, const std::string& callId2)
+Manager::joinParticipant(const std::string& callId1, const std::string& callId2, bool attached)
 {
     if (callId1 == callId2) {
         JAMI_ERR("Cannot join participant %s to itself", callId1.c_str());
@@ -1393,8 +1423,12 @@ Manager::joinParticipant(const std::string& callId1, const std::string& callId2)
     pimpl_->bindCallToConference(*call2, *conf);
 
     // Switch current call id to this conference
-    pimpl_->switchCall(conf->getConfID());
-    conf->setState(Conference::State::ACTIVE_ATTACHED);
+    if (attached) {
+        pimpl_->switchCall(conf->getConfID());
+        conf->setState(Conference::State::ACTIVE_ATTACHED);
+    } else {
+        conf->detach();
+    }
 
     pimpl_->conferenceMap_.emplace(conf->getConfID(), conf);
     emitSignal<DRing::CallSignal::ConferenceCreated>(conf->getConfID());
@@ -1437,30 +1471,48 @@ Manager::createConfFromParticipantList(const std::vector< std::string > &partici
     }
 }
 
-bool
-Manager::detachLocalParticipant()
+void
+Manager::setConferenceLayout(const std::string& confId, int layout)
 {
-    JAMI_DBG("Unbind local participant from conference");
-    auto conf = getConferenceFromID(getCurrentCallId());
-    if (not conf) {
-        JAMI_ERR("Current call id (%s) is not a conference", getCurrentCallId().c_str());
-        return false;
-    }
-
-    getRingBufferPool().unBindAll(RingBufferPool::DEFAULT_ID);
-
-    switch (conf->getState()) {
-        case Conference::State::ACTIVE_ATTACHED:
-            conf->setState(Conference::State::ACTIVE_DETACHED);
+    if (auto conf = getConferenceFromID(confId)) {
+        auto videoMixer = conf->getVideoMixer();
+        switch (layout)
+        {
+        case 0:
+            videoMixer->setVideoLayout(video::Layout::GRID);
+            break;
+        case 1:
+            videoMixer->setVideoLayout(video::Layout::ONE_BIG_WITH_SMALL);
+            break;
+        case 2:
+            videoMixer->setVideoLayout(video::Layout::ONE_BIG);
             break;
         default:
-            JAMI_WARN("Undefined behavior, invalid conference state in detach participant");
+            break;
+        }
     }
+}
 
-    emitSignal<DRing::CallSignal::ConferenceChanged>(conf->getConfID(), conf->getStateStr());
+void
+Manager::setActiveParticipant(const std::string& confId, const std::string& callId)
+{
+    if (auto conf = getConferenceFromID(confId)) {
+        conf->setActiveParticipant(callId);
+    }
+}
 
-    pimpl_->unsetCurrentCall();
-    return true;
+bool
+Manager::detachLocalParticipant(const std::string& conf_id)
+{
+    JAMI_DBG("Unbind local participant from conference");
+    if (auto conf = getConferenceFromID(conf_id.empty() ? getCurrentCallId() : conf_id)) {
+        conf->detach();
+        emitSignal<DRing::CallSignal::ConferenceChanged>(conf->getConfID(), conf->getStateStr());
+        pimpl_->unsetCurrentCall();
+        return true;
+    }
+    JAMI_ERR("Current call id (%s) is not a conference", getCurrentCallId().c_str());
+    return false;
 }
 
 bool
@@ -1679,6 +1731,9 @@ Manager::saveConfig()
         audioPreference.serialize(out);
 #ifdef ENABLE_VIDEO
         videoPreferences.serialize(out);
+#endif
+#ifdef ENABLE_PLUGIN
+        pluginPreferences.serialize(out);
 #endif
         shortcutPreferences.serialize(out);
 
@@ -1796,8 +1851,42 @@ Manager::incomingCall(Call &call, const std::string& accountId)
     emitSignal<DRing::CallSignal::IncomingCall>(accountId, callID, call.getPeerDisplayName() + " " + from);
 
     auto currentCall = getCurrentCall();
-    if (pimpl_->autoAnswer_) {
-        runOnMainThread([this, callID]{ answerCall(callID); });
+    if (call.getAccount().isRendezVous()) {
+        runOnMainThread([this, callID] {
+            answerCall(callID);
+            auto call = getCallFromCallID(callID);
+            auto accountId = call->getAccountId();
+            for (const auto& cid: getCallList()) {
+                if (auto call = getCallFromCallID(cid)) {
+                    if (call->getState() != Call::CallState::ACTIVE)
+                        continue;
+                    if (call->getAccountId() == accountId) {
+                        if (cid != callID) {
+                            if (call->getConfId().empty()) {
+                                joinParticipant(callID, cid, false);
+                            } else {
+                                addParticipant(callID, call->getConfId());
+                            }
+                            return;
+                        }
+                    }
+                }
+            }
+            // First call
+            auto conf = std::make_shared<Conference>();
+
+            // Bind calls according to their state
+            pimpl_->bindCallToConference(*call, *conf);
+            conf->detach();
+
+            pimpl_->conferenceMap_.emplace(conf->getConfID(), conf);
+            emitSignal<DRing::CallSignal::ConferenceCreated>(conf->getConfID());
+
+        });
+    } else if (pimpl_->autoAnswer_) {
+        runOnMainThread([this, callID]{
+            answerCall(callID);
+        });
     } else if (currentCall) {
         // Test if already calling this person
         if (currentCall->getAccountId() == accountId
@@ -1854,8 +1943,9 @@ Manager::incomingMessage(const std::string& callID,
 
         // in case of a conference we must notify client using conference id
         emitSignal<DRing::CallSignal::IncomingMessage>(conf->getConfID(), from, messages);
-    } else
+    } else {
         emitSignal<DRing::CallSignal::IncomingMessage>(callID, from, messages);
+    }
 }
 
 void
@@ -1922,7 +2012,7 @@ Manager::peerRingingCall(Call& call)
 {
     JAMI_DBG("[call:%s] Peer ringing", call.getCallId().c_str());
 
-    if (isCurrentCall(call))
+    if (!hasCurrentCall())
         ringback();
 }
 
@@ -2008,7 +2098,7 @@ Manager::stopTone()
 void
 Manager::playTone()
 {
-    pimpl_->playATone(Tone::TONE_DIALTONE);
+    pimpl_->playATone(Tone::ToneId::DIALTONE);
 }
 
 /**
@@ -2017,7 +2107,7 @@ Manager::playTone()
 void
 Manager::playToneWithMessage()
 {
-    pimpl_->playATone(Tone::TONE_CONGESTION);
+    pimpl_->playATone(Tone::ToneId::CONGESTION);
 }
 
 /**
@@ -2026,7 +2116,7 @@ Manager::playToneWithMessage()
 void
 Manager::congestion()
 {
-    pimpl_->playATone(Tone::TONE_CONGESTION);
+    pimpl_->playATone(Tone::ToneId::CONGESTION);
 }
 
 /**
@@ -2035,7 +2125,7 @@ Manager::congestion()
 void
 Manager::ringback()
 {
-    pimpl_->playATone(Tone::TONE_RINGTONE);
+    pimpl_->playATone(Tone::ToneId::RINGTONE);
 }
 
 /**
@@ -2733,6 +2823,15 @@ Manager::loadAccountMap(const YAML::Node& node)
         shortcutPreferences.unserialize(node);
 #ifdef ENABLE_VIDEO
         videoPreferences.unserialize(node);
+#endif
+#ifdef ENABLE_PLUGIN
+        pluginPreferences.unserialize(node);
+
+        std::vector<std::string> loadedPlugins = pluginPreferences.getLoadedPlugins();
+        for (const std::string& plugin : loadedPlugins)
+        {
+            jami::Manager::instance().getJamiPluginManager().loadPlugin(plugin);
+        }
 #endif
     } catch (const YAML::Exception &e) {
         JAMI_ERR("%s: Preferences node unserialize error: ", e.what());
@@ -2792,7 +2891,7 @@ Manager::loadAccountMap(const YAML::Node& node)
 }
 
 std::map<std::string, std::string>
-Manager::getCallDetails(const std::string &callID)
+Manager::getCallDetails(const std::string &callID) const
 {
     if (auto call = getCallFromCallID(callID)) {
         return call->getDetails();
@@ -2814,6 +2913,16 @@ Manager::getCallList() const
     return results;
 }
 
+std::vector<std::map<std::string, std::string>>
+Manager::getConferenceInfos(const std::string& confId) const
+{
+    if (auto conf = getConferenceFromID(confId))
+        return conf->getConferenceInfos();
+    else if (auto call = getCallFromCallID(confId))
+        return call->getConferenceInfos();
+    return {};
+}
+
 std::map<std::string, std::string>
 Manager::getConferenceDetails(const std::string& confID) const
 {
@@ -2920,13 +3029,33 @@ Manager::sendRegister(const std::string& accountID, bool enable)
         acc->doUnregister();
 }
 
+bool
+Manager::isPasswordValid(const std::string& accountID, const std::string& password)
+{
+    const auto acc = getAccount<JamiAccount>(accountID);
+    if (!acc)
+        return false;
+    return acc->isPasswordValid(password);
+}
+
 uint64_t
 Manager::sendTextMessage(const std::string& accountID, const std::string& to,
                          const std::map<std::string, std::string>& payloads)
 {
     if (const auto acc = getAccount(accountID)) {
         try {
+#ifdef ENABLE_PLUGIN
+            auto& convManager = jami::Manager::instance().getJamiPluginManager()
+                    .getConversationServicesManager();
+            std::shared_ptr<jami::ConversationMessage> cm =
+                    std::make_shared<jami::ConversationMessage>(accountID, to,
+                                                                const_cast<std::map<std::string,
+                                                                std::string>&>(payloads));
+            convManager.sendTextMessage(cm);
+            return acc->sendTextMessage(cm->to_, cm->data_);
+#else
             return acc->sendTextMessage(to, payloads);
+#endif //ENABLE_PLUGIN
         } catch (const std::exception& e) {
             JAMI_ERR("Exception during text message sending: %s", e.what());
         }
@@ -3070,6 +3199,20 @@ Manager::getLastMessages(const std::string& accountID, const uint64_t& base_time
     return {};
 }
 
+SIPVoIPLink&
+Manager::sipVoIPLink() const
+{
+    return *pimpl_->sipLink_;
+}
+
+#ifdef ENABLE_PLUGIN
+JamiPluginManager&
+Manager::getJamiPluginManager() const
+{
+    return pimpl_->jami_plugin_manager;
+}
+#endif
+
 std::map<std::string, std::string>
 Manager::getNearbyPeers(const std::string& accountID)
 {
